-- Big Assembly prototype and item definition

local function create_bigassembly(name, energy, speed)
    local bigassembly = util.table.deepcopy(data.raw["assembling-machine"]["assembling-machine-3"])
    local icon = "__WhistleStopFactories__/graphics/icons/big-assembly.png"

    bigassembly.name = name
    bigassembly.icon = icon
    bigassembly.localised_name = {"entity-name.big-assembly"}

    bigassembly.minable = nil
    bigassembly.fast_replaceable_group = nil
    bigassembly.dying_explosion = "big-explosion"

    bigassembly.collision_box = {{-8.1, -8.1}, {8.1, 8.1}}
    bigassembly.selection_box = {{-8.8, -9}, {8.8, 9}}
    bigassembly.drawing_box = {{-8.8, -8.8}, {8.8, 8.8}}

    bigassembly.crafting_categories = {"big-recipe", "big-chem"}
    bigassembly.crafting_speed = speed

    bigassembly.energy_usage = energy
    bigassembly.ingredient_count = 10
    bigassembly.module_specification.module_slots = 5
    bigassembly.map_color = {r=103, g=247, b=247}
    bigassembly.scale_entity_info_icon = true

    bigassembly.create_ghost_on_death = false
    -- Normal flags for assembly machine: {"placeable-neutral", "placeable-player", "player-creation"}
    bigassembly.flags = {"placeable-neutral", "placeable-player", "player-creation", "not-deconstructable", "not-blueprintable"}
    bigassembly.collision_mask = bigassembly.collision_mask or {"item-layer", "object-layer", "player-layer", "water-tile"}
    table.insert(bigassembly.collision_mask, "resource-layer")
    table.insert(bigassembly.resistances, {percent=100, type="poison"})  -- Prevent termite damage

    local function fluidBox(type, position)
        retvalue = {
                production_type = type,
                pipe_picture = assembler3pipepictures(),
                pipe_covers = pipecoverspictures(),
                base_area = 10,
                pipe_connections = {{ type=type, position=position }},
                secondary_draw_orders = { north = -1 }
            }
        if type == "input" then
            retvalue.base_level = -1
        else
            retvalue.base_level = 1
        end
        return retvalue
    end

    bigassembly.fluid_boxes = {
        fluidBox("input", {1, -9}),
        fluidBox("input", {-9, -1}),
        fluidBox("output", {9, 1}),
        fluidBox("output", {-1, 9}),
        -- off_when_no_fluid_recipe = true -- Allows for rotation
    }

    -- Scale graphics by a factor and correct animation speed
    local function bumpUp(animation, factor)
        animation.shift = util.table.deepcopy(animation.shift)
        animation.shift[1] = animation.shift[1] * factor   
        animation.shift[2] = animation.shift[2] * factor

        animation.scale = (animation.scale or 1) * factor
        animation.animation_speed = 0.05
    end

    local scaleFactor = 6
    for k,v in pairs(bigassembly.animation.layers) do
        bumpUp(v, scaleFactor)
        bumpUp(v.hr_version, scaleFactor)
    end

    data.raw["assembling-machine"][name] = bigassembly

    local bigassembly_item = util.table.deepcopy(data.raw.item["assembling-machine-3"])

    bigassembly_item.name = name
    bigassembly_item.icon = icon
    bigassembly_item.order = "c[" .. name .. "]"
    bigassembly_item.place_result = name

    data.raw.item[name] = bigassembly_item
end

return create_bigassembly