script.on_event(defines.events.on_player_created, function(event)
    local miningSpeedMultiplier = settings.startup["early-riser-mining-speed-multiplier"].value
    local craftingSpeedModifier = settings.startup["early-riser-crafting-speed-multiplier"].value
    local fastStart = settings.startup["early-riser-fast-start"].value
    local fastStartPower = settings.startup["early-riser-fast-start-power"].value
    local fastStartBase = settings.startup["early-riser-fast-start-base"].value
    local revealSize = settings.global["early-riser-reveal-size"].value
    local alwaysDay = settings.global["early-riser-daytime"].value

    local player = game.players[event.player_index]
    local color_table = {
        ["red"] = {r=1,g=0,b=0,a=0.9},
        ["orange"] = {r=1,g=0.65,b=0,a=0.9},
        ["yellow"] = {r=1,g=1,b=0,a=0.9},
        ["green"] = {r=0,g=1,b=0,a=0.9},
        ["blue"] = {r=0,g=0,b=1,a=0.9},
        ["indigo"] = {r=0.29,g=0,b=0.51,a=0.9},
        ["violet"] = {r=0.93,g=0.51,b=0.93,a=0.9},
        ["black"] = {r=1,g=1,b=1,a=0.9},
        ["white"] = {r=0,g=0,b=0,a=0.9}
    }
    player.color = color_table[settings.get_player_settings(player)["early-riser-player-color"].value]    
    if alwaysDay then
        game.surfaces[1].always_day = true
    end
	if fastStart then
        local chestFast = player.surface.create_entity{name = "steel-chest", position = {player.position.x+1, player.position.y}, force=player.force}
        if game.active_mods["Nanobots"] then
            chestFast.insert{name="gun-nano-emitter", count = 2}
            chestFast.insert{name="ammo-nano-constructors", count = 1000}
            chestFast.insert{name="ammo-nano-termites", count = 1000}
        end
        chestFast.insert{name="electric-mining-drill", count = 100}
        chestFast.insert{name="transport-belt", count = 200}
        chestFast.insert{name="small-electric-pole", count = 100}
        chestFast.insert{name="underground-belt", count = 20}
    end
    if fastStartPower then
        local chestPower = player.surface.create_entity{name = "steel-chest", position = player.position, force=player.force}
        chestPower.insert{name="offshore-pump", count = 1}
        chestPower.insert{name="pipe", count = 100}
        chestPower.insert{name="pipe-to-ground", count = 50}
        chestPower.insert{name="boiler", count = 20}
        chestPower.insert{name="steam-engine", count = 40}
        chestPower.insert{name="burner-inserter", count = 20}
        chestPower.insert{name="small-electric-pole", count = 100}
        chestPower.insert{name="transport-belt", count = 500}
        chestPower.insert{name="splitter", count = 50}
    end
    if fastStartBase then
        local chestBase = player.surface.create_entity{name = "steel-chest", position = {player.position.x, player.position.y+1}, force=player.force}
        chestBase.insert{name="transport-belt", count = 500}
        chestBase.insert{name="inserter", count = 147}
        chestBase.insert{name="small-electric-pole", count = 51}
        chestBase.insert{name="stone-furnace", count = 50}
        chestBase.insert{name="lab", count = 25}
        chestBase.insert{name="assembling-machine-1", count = 18}
        chestBase.insert{name="small-lamp", count = 8}
        chestBase.insert{name="iron-chest", count = 4}
        chestBase.insert{name="fast-inserter", count = 4}
        chestBase.insert{name="assembling-machine-2", count = 3}
        chestBase.insert{name="filter-inserter", count = 3}
        chestBase.insert{name="wooden-chest", count = 2}
        chestBase.insert{name="underground-belt", count = 2}
        chestBase.insert{name="splitter", count = 2}
    end
    player.force.manual_mining_speed_modifier = miningSpeedMultiplier
    player.force.manual_crafting_speed_modifier = craftingSpeedModifier
    player.force.chart(player.surface, {{player.position.x-revealSize, player.position.y-revealSize}, {player.position.x+revealSize, player.position.y+revealSize}})
end)

script.on_event(defines.events.on_research_finished, function(event)
  if event.research.name == "nanobots" or event.research.name == "construction-robotics" then
    for i, player in pairs(event.research.force.players) do
      event.research.force.manual_mining_speed_modifier = 0
      event.research.force.manual_crafting_speed_modifier = 0
    end
  end
  if event.research.name == "logistics" then
    for i, player in pairs(event.research.force.players) do
      event.research.force.recipes["loader"].enabled = settings.startup["early-riser-loaders"].value
    end
  end
  if event.research.name == "logistics-2" then
    for i, player in pairs(event.research.force.players) do
      event.research.force.recipes["fast-loader"].enabled = settings.startup["early-riser-loaders"].value
    end
  end
  if event.research.name == "logistics-3" then
    for i, player in pairs(event.research.force.players) do
      event.research.force.recipes["express-loader"].enabled = settings.startup["early-riser-loaders"].value
    end
  end
end)

script.on_event(defines.events.on_runtime_mod_setting_changed, function(event)
  if event.setting_type == "runtime-global" then
    for i, player in pairs(game.players) do
        if event.setting == "early-riser-daytime" then
			game.surfaces[1].always_day = settings.global["early-riser-daytime"].value
        end
        if event.setting == "early-riser-reveal-size" then
            local revealSize = settings.global["early-riser-reveal-size"].value
			player.force.chart(player.surface, {{player.position.x-revealSize, player.position.y-revealSize}, {player.position.x+revealSize, player.position.y+revealSize}})
        end
    end
  end
  local player = game.players[event.player_index]
  
  if event.setting == "early-riser-player-color" then
    local color_table = {
        ["red"] = {r=1,g=0,b=0,a=0.9},
        ["orange"] = {r=1,g=0.65,b=0,a=0.9},
        ["yellow"] = {r=1,g=1,b=0,a=0.9},
        ["green"] = {r=0,g=1,b=0,a=0.9},
        ["blue"] = {r=0,g=0,b=1,a=0.9},
        ["indigo"] = {r=0.29,g=0,b=0.51,a=0.9},
        ["violet"] = {r=0.93,g=0.51,b=0.93,a=0.9},
        ["black"] = {r=1,g=1,b=1,a=0.9},
        ["white"] = {r=0,g=0,b=0,a=0.9}
    }
    player.color = color_table[settings.get_player_settings(player)["early-riser-player-color"].value]    
  end
end)
