--[[This file creates the loader entities in the game, creates characteristics using the 
	appropriate-tier transport belt as a base.]]--

-- Create color masks
local tint_mask = {
	["basic-loader"]   = {r = 222, g = 222, b = 222}, -- Corrected 6-12-2018 11:05 AM
	["loader"] 		   = {r = 245, g = 198, b =  56}, -- Corrected 6-12-2018 11:03 AM
	["fast-loader"]    = {r = 242, g =  48, b =  61}, -- Corrected 6-12-2018 11:02 AM
	["express-loader"] = {r = 115, g = 193, b = 229}, -- Corrected 6-12-2018 11:07 AM
	["purple-loader"]  = {r = 168, g =  47, b = 237}, -- Corrected 6-12-2018 11:41 AM
	["green-loader"]   = {r =  55, g = 240, b =  86}, -- Corrected 6-12-2018 11:49 AM
}

local hr_tint_mask = {
	["basic-loader"]   = {r = 222, g = 222, b = 222}, -- Corrected 6-10-2018  2:05 PM
	["loader"] 		   = {r = 229, g = 187, b =  46}, -- Corrected 6-10-2018 12:24 PM
	["fast-loader"]    = {r = 229, g =  37, b =  37}, -- Corrected 6-10-2018  1:48 PM
	["express-loader"] = {r =  78, g = 192, b = 245}, -- Corrected 6-10-2018  1:54 PM
	["purple-loader"]  = {r = 197, g =  76, b = 245}, -- Corrected 6-10-2018  2:00 PM
	["green-loader"]   = {r =  77, g = 229, b =  34}, -- Corrected 6-10-2018  2:04 PM
}
	
-- Called by make_loader_entity. Creates the icon, and low/high resolution graphics.
local function patch_loader_entity(loader)
	if loader then
		local name = loader.name
		-- loader.icon = "__vanilla-loaders-hd__/graphics/icons/"..name..".png"
		-- loader.fast_replaceable_group = "loader"
		
		loader.structure.direction_in.sheets = 
		{
			{
				filename = "__vanilla-loaders-hd__/graphics/entity/loader/loader-structure-base.png",				
				width    = 96,
				height   = 64,
				y        = 0,
				hr_version = 
				{
					filename = "__vanilla-loaders-hd__/graphics/entity/loader/hr-loader-structure-base.png",
					height   = 128,
					priority = "extra-high",
					scale    = 0.5,
					width    = 192,
					y        = 0
				}
			},
			{
				filename = "__vanilla-loaders-hd__/graphics/entity/loader/loader-structure-mask.png",			
				width    = 96,
				height   = 64,
				y        = 0,
				tint	 = tint_mask[name],
				hr_version = 
				{
					filename = "__vanilla-loaders-hd__/graphics/entity/loader/hr-loader-structure-mask.png",
					height   = 128,
					priority = "extra-high",
					scale    = 0.5,
					width    = 192,
					y        = 0,
					tint     = hr_tint_mask[name]
				}
			}
		}
		loader.structure.direction_out.sheets = 
		{
			{
				filename = "__vanilla-loaders-hd__/graphics/entity/loader/loader-structure-base.png",			
				width    = 96,
				height   = 64,
				y        = 64,
				hr_version = 
				{
					filename = "__vanilla-loaders-hd__/graphics/entity/loader/hr-loader-structure-base.png",
					height   = 128,
					priority = "extra-high",
					scale    = 0.5,
					width    = 192,
					y        = 128
				}
			},
			{
				filename = "__vanilla-loaders-hd__/graphics/entity/loader/loader-structure-mask.png",			
				width    = 96,
				height   = 64,
				y        = 64,
				tint	 = tint_mask[name],
				hr_version = 
				{
					filename = "__vanilla-loaders-hd__/graphics/entity/loader/hr-loader-structure-mask.png",
					height   = 128,
					priority = "extra-high",
					scale    = 0.5,
					width    = 192,
					y        = 128,
					tint     = hr_tint_mask[name]
				}
			}
		}
	end
end

-- Create new loader entities.
local function make_loader_entity(name, beltname)
	if data.raw["transport-belt"][beltname] then
		local loader = table.deepcopy(data.raw["loader"]["loader"])
		local basebelt = data.raw["transport-belt"][beltname]
		loader.name            = name
		-- loader.icon            = "__vanilla-loaders-hd__/graphics/icons/"..name..".png"
		loader.minable.result  = name
		loader.belt_horizontal = basebelt.belt_horizontal
		loader.belt_vertical   = basebelt.belt_vertical
		loader.ending_top      = basebelt.ending_top
		loader.ending_bottom   = basebelt.ending_bottom
		loader.ending_side     = basebelt.ending_side
		loader.starting_top    = basebelt.starting_top
		loader.starting_bottom = basebelt.starting_bottom
		loader.starting_side   = basebelt.starting_side
		loader.speed           = basebelt.speed
		patch_loader_entity(loader)
		data:extend({loader})
	end
end

-- Check for Loader Redux, and if we're being compliant. If both are true, then let Loader Redux handle everything
-- and just patch the existing items with our graphics.
if mods["LoaderRedux"] and settings.startup["vanillaLoaders-recipes-loaderReduxCompliant"].value == true then

	-- Just reskin everything
	patch_loader_entity(data.raw["loader"]["loader"])
	patch_loader_entity(data.raw["loader"]["fast-loader"])
	patch_loader_entity(data.raw["loader"]["express-loader"])
	patch_loader_entity(data.raw["loader"]["purple-loader"])
	patch_loader_entity(data.raw["loader"]["green-loader"])

	-- Since Loader Redux does not have a basic loader, see if we want to make one.
	if settings.startup["vanillaLoaders-recipes-loaderReduxBasicLoader"].value == true then 
		-- Make the basic loader
		make_loader_entity("basic-loader","basic-transport-belt")
	end

else

	-- Vanilla Loader patching
	patch_loader_entity(data.raw["loader"]["loader"])
	patch_loader_entity(data.raw["loader"]["fast-loader"])
	patch_loader_entity(data.raw["loader"]["express-loader"])

	-- When Bob's Logistics is present.
	if mods["boblogistics"] then
		-- Create turbo/ultimate loaders when Bob's logistics is present.
		make_loader_entity("purple-loader","turbo-transport-belt")
		make_loader_entity("green-loader","ultimate-transport-belt")

		-- If Bob's Logistics belt overhaul is enabled, add the basic loader.
		if settings.startup["bobmods-logistics-beltoverhaul"].value == true then
			
			-- Make the basic loader
			make_loader_entity("basic-loader","basic-transport-belt")
		end
	end
end
