-- Create color masks
local tint_mask = {
	["basic-loader"]   = {r = 222, g = 222, b = 222}, -- Corrected 6-12-2018 11:05 AM
	["loader"] 		   = {r = 245, g = 198, b =  56}, -- Corrected 6-12-2018 11:03 AM
	["fast-loader"]    = {r = 242, g =  48, b =  61}, -- Corrected 6-12-2018 11:02 AM
	["express-loader"] = {r = 115, g = 193, b = 229}, -- Corrected 6-12-2018 11:07 AM
	["purple-loader"]  = {r = 168, g =  47, b = 237}, -- Corrected 6-12-2018 11:41 AM
	["green-loader"]   = {r =  55, g = 240, b =  86}, -- Corrected 6-12-2018 11:49 AM
}

-- Patch loaders to comport with vanilla aesthetic.
local function patch_loader_item(item, beltname, subgroup)
	if item then
		local baseitem = data.raw["item"][beltname]
		local name = item.name
				
		-- Optional parameter
		subgroup = subgroup or baseitem.subgroup
		
		-- Main function
		item.icons  = 
		{
			{
				icon = "__vanilla-loaders-hd__/graphics/icons/loader-icon-base.png"
			},
			{
				icon = "__vanilla-loaders-hd__/graphics/icons/loader-icon-mask.png",
				tint = tint_mask[name]
			}
		}
		item.flags = {"goes-to-quickbar"}
		
		-- [Disabled]
		-- Check for ShinyIcons to sort appropriately in the case of the basic loader.
		-- Really extra solution, but pending support in ShinyIcons I don't have a better one.
		--if mods["ShinyIcons"] and beltname == "basic-transport-belt" then
		--	item.order = "a0"
		--	item.subgroup = "shinybelt4"
		--else
			item.subgroup = subgroup
			item.order = string.gsub(string.gsub(item.order,"^[a-z]","d"),"transport%-belt","loader")
		--end
	end
end

-- Create default-compliant loaders, consistent with vanilla loaders and aesthetic. 
-- Used for Bob's-style loaders.
local function make_loader_item(name, beltname, subgroup)
	if data.raw["item"][beltname] then
		local baseitem = data.raw["item"][beltname]
		local item = table.deepcopy(data.raw["item"]["loader"])
		item.name = name
		item.place_result = name
		patch_loader_item(item, beltname, subgroup)
		data:extend({item})
	end
end

-- Check for Loader Redux, and if we're being compliant. If both are true, then let Loader Redux handle everything
-- and just patch the existing items with our graphics.
if mods["LoaderRedux"] and settings.startup["vanillaLoaders-recipes-loaderReduxCompliant"].value == true then

	if mods["boblogistics"] then
		-- Just reskin everything
		patch_loader_item(data.raw["item"]["loader"],"transport-belt","bob-logistic-tier-1")
		patch_loader_item(data.raw["item"]["fast-loader"],"fast-transport-belt","bob-logistic-tier-2")
		patch_loader_item(data.raw["item"]["express-loader"],"express-transport-belt","bob-logistic-tier-3")
		patch_loader_item(data.raw["item"]["purple-loader"],"turbo-transport-belt","bob-logistic-tier-4")
		patch_loader_item(data.raw["item"]["green-loader"],"ultimate-transport-belt","bob-logistic-tier-5")

		-- Since Loader Redux does not have a basic loader, see if we want to make one.
		if settings.startup["vanillaLoaders-recipes-loaderReduxBasicLoader"].value == true then 
			-- Make the basic loader
			make_loader_item("basic-loader","basic-transport-belt")
		end
	else
		-- Just reskin everything
		patch_loader_item(data.raw["item"]["loader"],"transport-belt")
		patch_loader_item(data.raw["item"]["fast-loader"],"fast-transport-belt")
		patch_loader_item(data.raw["item"]["express-loader"],"express-transport-belt")
		patch_loader_item(data.raw["item"]["purple-loader"],"turbo-transport-belt")
		patch_loader_item(data.raw["item"]["green-loader"],"ultimate-transport-belt")
	end

else

	-- When Bob's Logistics is present.
	if mods["boblogistics"] then

		-- Patch the vanilla loaders, sort with Bob's transport subgroups
		patch_loader_item(data.raw["item"]["loader"],"transport-belt","bob-logistic-tier-1")
		patch_loader_item(data.raw["item"]["fast-loader"],"fast-transport-belt","bob-logistic-tier-2")
		patch_loader_item(data.raw["item"]["express-loader"],"express-transport-belt","bob-logistic-tier-3")
		
		-- Create the Bob's style loaders
		make_loader_item("purple-loader","turbo-transport-belt")
		make_loader_item("green-loader","ultimate-transport-belt")

		-- If Bob's Logistics belt overhaul is enabled, create the basic loader.
		if settings.startup["bobmods-logistics-beltoverhaul"].value == true then
			make_loader_item("basic-loader","basic-transport-belt")
		end
		
	-- When Bob's Logistics is not present.
	else
		-- Patch the vanilla loaders
		patch_loader_item(data.raw["item"]["loader"],"transport-belt")
		patch_loader_item(data.raw["item"]["fast-loader"],"fast-transport-belt")
		patch_loader_item(data.raw["item"]["express-loader"],"express-transport-belt")
	end
end