-- Create color masks
local tint_mask = {
	["basic-loader"]   = {r = 222, g = 222, b = 222}, -- Corrected 6-12-2018 11:05 AM
	["loader"] 		   = {r = 245, g = 198, b =  56}, -- Corrected 6-12-2018 11:03 AM
	["fast-loader"]    = {r = 242, g =  48, b =  61}, -- Corrected 6-12-2018 11:02 AM
	["express-loader"] = {r = 115, g = 193, b = 229}, -- Corrected 6-12-2018 11:07 AM
	["purple-loader"]  = {r = 168, g =  47, b = 237}, -- Corrected 6-12-2018 11:41 AM
	["green-loader"]   = {r =  55, g = 240, b =  86}, -- Corrected 6-12-2018 11:49 AM
}

-- Function to add the loaders to the technology tree.
local function patch_loader_technology(technology, recipe)
	if data.raw["technology"][technology] then
		table.insert(data.raw["technology"][technology].effects, {
			type="unlock-recipe",
			recipe=recipe
		})
	end
end

-- Function to create vanilla-style tech icons
local function patch_tech_icon(loader)
	if data.raw["technology"][loader] then
		data.raw["technology"][loader].icons = 
		{
			{
				icon = "__vanilla-loaders-hd__/graphics/technology/loader-tech-base.png"
			},
			{
				icon = "__vanilla-loaders-hd__/graphics/technology/loader-tech-mask.png",
				tint = tint_mask[loader]
			}
		}
	end
end

-- Check for presence of LoaderRedux, and check if we are supposed to use LoaderRedux research. 
-- If true, stop here, reskin the technologies to vanilla-style, and if needed create basic loader technology.
if mods["LoaderRedux"] and settings.startup["vanillaLoaders-research-loaderReduxCompliant-revised"].value == true then -- Yes LoaderRedux, Yes Research

	-- Check if we need a basic loader tech, and add it if we do.
	if data.raw["loader"]["basic-loader"] then
		data:extend({
			{
			type = "technology",
			name = "basic-loader",
			icons = 
			{
				{
					icon = "__vanilla-loaders-hd__/graphics/technology/loader-tech-base.png"
				},
				{
					icon = "__vanilla-loaders-hd__/graphics/technology/loader-tech-mask.png",
					tint = tint_mask["basic-loader"]
				}
			},
			icon_size = 128,
			effects =
			{
			{
				type = "unlock-recipe",
				recipe = "basic-loader"
			},
			},
			prerequisites = {"bob-logistics-0"},
			unit =
			{
			count = 20,
			ingredients =
			{
				{"science-pack-1", 1}
			},
			time = 15
			},
			order = "a-f-a",
			}
		})
	end
	
	-- Reskin the existing Loader Redux techs
	patch_tech_icon("loader")
	patch_tech_icon("fast-loader")
	patch_tech_icon("express-loader")
	patch_tech_icon("purple-loader")
	patch_tech_icon("green-loader")

	return

end

-- If we got this far, we didn't want LoaderRedux research.
-- Add the vanilla loaders to the technology tree.
patch_loader_technology("logistics"      ,"loader"        		) -- Yellow
patch_loader_technology("logistics-2"    ,"fast-loader"   		) -- Red
patch_loader_technology("logistics-3"    ,"express-loader"		) -- Blue

-- When Bob's Logistics is present.
if mods["boblogistics"] then
	-- Add Bob's loaders to the technology tree.
	patch_loader_technology("bob-logistics-4","purple-loader"	) -- Purple
	patch_loader_technology("bob-logistics-5","green-loader" 	) -- Green

	-- Add the Basic Loader to the tech tree
	if data.raw["loader"]["basic-loader"] then
		patch_loader_technology("bob-logistics-0","basic-loader") -- Black
	end
end

-- Check for and remove Loader Redux technologies.
if data.raw.technology["loader"] then
	data.raw.technology["loader"] = null
end

if data.raw.technology["fast-loader"] then
	data.raw.technology["fast-loader"] = null
end

if data.raw.technology["express-loader"] then
	data.raw.technology["express-loader"] = null
end

if data.raw.technology["purple-loader"] then
	data.raw.technology["purple-loader"] = null
end

if data.raw.technology["green-loader"] then
	data.raw.technology["green-loader"] = null
end